@@ -3,11 +3,11 @@
 ;; Copyright (C) 2006-2009, 2011-2012, 2015-2018
 ;;   Phil Hagelberg, Doug Alcorn, Will Farrington, Chen Bin
 ;;
-;; Version: 5.7.10
+;; Version: 6.0.0
 ;; Author: Phil Hagelberg, Doug Alcorn, and Will Farrington
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
 ;; URL: https://github.com/technomancy/find-file-in-project
-;; Package-Requires: ((ivy "0.10.0") (emacs "24.4"))
+;; Package-Requires: ((emacs "25.1"))
 ;; Created: 2008-03-18
 ;; Keywords: project, convenience
 ;; EmacsWiki: FindFileInProject
@@ -36,6 +36,7 @@
 ;; - Works on Windows with minimum setup
 ;; - Works on Tramp Mode (https://www.emacswiki.org/emacs/TrampMode)
 ;; - fd (faster alternative of find, see https://github.com/sharkdp/fd) is supported
+;; - Uses only native API `completing-read' and supports ido/ivy/helm out of box
 ;;
 ;; Usage,
 ;;   - You can insert "(setq ffip-use-rust-fd t)" into ".emacs" to use fd (alternative of find)
@@ -51,6 +52,8 @@
 ;;   - `ffip-create-project-file' creates ".dir-locals.el"
 ;;   - `ffip-lisp-find-file-in-project' finds file in project.
 ;;     If its parameter is not nil, it find directory.
+;;     This command is written in pure Lisp and does not use any third party
+;;     command line program.  So it works in all environments.
 ;;
 ;; A project is found by searching up the directory tree until a file
 ;; is found that matches `ffip-project-file'.
@@ -95,9 +98,8 @@
 ;; To find in current directory, use `find-file-in-current-directory'
 ;; and `find-file-in-current-directory-by-selected'.
 ;;
-;; `ffip-split-window-horizontally' and `ffip-split-window-vertically' find&open file
-;; in split window.
-
+;; `ffip-fix-file-path-at-point' replaces path at point with correct relative/absolute path.
+;;
 ;; `ffip-show-diff' execute the backend from `ffip-diff-backends'.
 ;; The output is in Unified Diff Format and inserted into *ffip-diff* buffer.
 ;; Press "o" or "C-c C-c" or "ENTER" or `M-x ffip-diff-find-file' in the
@@ -126,19 +128,7 @@
 ;; to format the relative path,
 ;;   (setq ffip-find-relative-path-callback 'ffip-copy-reactjs-import)
 ;;   (setq ffip-find-relative-path-callback 'ffip-copy-org-file-link)
-
-;; `ivy-mode' is used for filter/search UI
-;; In `ivy-mode', SPACE is translated to regex ".*".
-;; For example, the search string "dec fun pro" is transformed into
-;; regular expression "\\(dec\\).*\\(fun\\).*\\(pro\\)"
-;; `C-h i g (ivy)' for more key-binding tips.
 ;;
-;; `ffip-save-ivy-last' saves the most recent search result.
-;; `ffip-ivy-resume' re-use the save result. Both requires `ivy-mode'
-;; installed.
-;;
-;; You can switch to `ido-mode' by `(setq ffip-prefer-ido-mode t)'
-
 ;; BSD/GNU Find can be installed through Cygwin or MYSYS2 on Windows.
 ;; Executable is automatically detected. But you can manually specify
 ;; the executable location by insert below code into ".emacs",
@@ -157,7 +147,7 @@
 (require 'windmove)
 (require 'subr-x)
 
-(defvar ffip-use-rust-fd nil "Use use fd instead of find.")
+(defvar ffip-use-rust-fd nil "Use rust fd instead of find.")
 
 (defvar ffip-rust-fd-respect-ignore-files t
   "Don 't show search results from '.*ignore' files.")
@@ -165,15 +155,6 @@
 (defvar ffip-rust-fd-extra-opts ""
   "Rust fd extra options passed to cli.")
 
-(defvar ffip-window-ratio-alist
-  '((1 . 1.61803398875)
-    (2 . 2)
-    (3 . 3)
-    (4 . 4)
-    (5 . 0.61803398875))
-  "Dictionary to look up windows split ratio.
-Used by `ffip-split-window-horizontally' and `ffip-split-window-vertically'.")
-
 (defvar ffip-filename-history nil)
 
 (defvar ffip-strip-file-name-regex
@@ -181,9 +162,6 @@ Used by `ffip-split-window-horizontally' and `ffip-split-window-vertically'.")
   "Strip file name to get minimum keyword with this regex.
 It's used by `find-file-with-similar-name'.")
 
-(defvar ffip-split-window-without-asking-for-keyword nil
-  "`ffip-split-window-horizontally' or `ffip-split-window-vertically' don't ask keyword.")
-
 (defvar ffip-diff-find-file-before-hook nil
   "Hook before `ffip-diff-find-file' move focus out of *ffip-diff* buffer.")
 
@@ -219,7 +197,7 @@ The file path is passed to the hook as the first argument.")
 (defun ffip-git-diff-current-file ()
   "Run 'git diff version:current-file current-file'."
   (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (ivy-read "diff current file:" (ffip-diff-git-versions))))
+         (line (completing-read "diff current file:" (ffip-diff-git-versions))))
     (ffip-shell-command-to-string (format "git --no-pager diff %s:%s %s"
                                      (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))
                                      (file-relative-name buffer-file-name default-directory)
@@ -228,7 +206,7 @@ The file path is passed to the hook as the first argument.")
 (defun ffip-git-diff-project()
   "Run 'git diff version' in project."
   (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (ivy-read "diff current file:" (ffip-diff-git-versions)))
+         (line (completing-read "diff current file:" (ffip-diff-git-versions)))
          (version (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))))
     (ffip-shell-command-to-string (format "git --no-pager diff %s" version))))
 
@@ -257,9 +235,6 @@ The output is inserted into *ffip-diff* buffer.")
   "The file/directory used to locate project root.
 May be set using .dir-locals.el.  Checks each entry if set to a list.")
 
-(defvar ffip-prefer-ido-mode (not (require 'ivy nil t))
-  "Use ido instead of ivy to display candidates.")
-
 (defvar ffip-patterns nil
   "List of glob patterns to look for with `find-file-in-project'.")
 
@@ -362,7 +337,7 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
 Use this to exclude portions of your project: \"-not -regex \\\".*svn.*\\\"\".")
 
 (defvar ffip-find-pre-path-options ""
-  "Options for find porgram.
+  "Options for find program.
 
 GNU Find requires '-H', '-L', '-P', '-D' and `-O' appear before first path '.'.
 For example, use '-L' to follow symbolic links.")
@@ -372,12 +347,8 @@ For example, use '-L' to follow symbolic links.")
 
 (defvar ffip-project-root-function nil
   "If non-nil, this function is called to determine the project root.
-
 This overrides variable `ffip-project-root' when set.")
 
-(defvar ffip-ivy-last-saved nil
-  "Backup of `ivy-last'.  Requires ivy.")
-
 (defvar ffip-debug nil "Print debug information.")
 
 ;;;###autoload
@@ -439,30 +410,12 @@ This overrides variable `ffip-project-root' when set.")
            (insert-file-contents-literally file)
            (buffer-substring-no-properties (point-min) (point-max))) 'utf-8)))
 
-;;;###autoload
-(defun ffip-save-ivy-last ()
-  "Save `ivy-last' into `ffip-ivy-last-saved'.  Requires ivy."
-  (interactive)
-  (if (boundp 'ivy-last)
-      (setq ffip-ivy-last-saved ivy-last)
-    (message "Sorry. You need install `ivy-mode' first.")))
-
 ;;;###autoload
 (defun ffip-get-project-root-directory ()
   "Get the full path of project root directory."
   (if ffip-project-root (file-name-as-directory ffip-project-root)
     (ffip-project-root)))
 
-;;;###autoload
-(defun ffip-ivy-resume ()
-  "Wrapper of `ivy-resume'.  Resume the search saved at `ffip-ivy-last-saved'."
-  (interactive)
-  (let* ((ivy-last (if ffip-ivy-last-saved ffip-ivy-last-saved ivy-last))
-         (default-directory (ffip-get-project-root-directory)))
-    (if (fboundp 'ivy-resume)
-        (ivy-resume)
-      (message "Sorry. You need install `ivy-mode' first."))))
-
 ;;;###autoload
 (defun ffip-filename-identity (keyword)
   "Return identical KEYWORD."
@@ -610,44 +563,30 @@ If CHECK-ONLY is true, only do the check."
 (defun ffip-completing-read (prompt collection &optional action)
   "Read a string in minibuffer, with completion.
 
-PROMPT is a string with same format parameters in `ido-completing-read'.
+PROMPT is a string with same format parameters in `completing-read'.
 COLLECTION is a list of strings.
 
 ACTION is a lambda function to call after selecting a result.
 
 This function returns the selected candidate or nil."
-  (cond
-   ((and action (= 1 (length collection)))
-    ;; open file directly
-    (funcall action (car collection))
-    (car collection))
-   ;; If user prefer `ido-mode' or there is no ivy,
-   ;; use `ido-completing-read'.
-   ((or ffip-prefer-ido-mode (not (fboundp 'ivy-read)))
-    ;; friendly UI for ido
-    (let* ((list-of-pair (consp (car collection)))
-           (ido-collection (if list-of-pair
-                               (mapcar 'car collection)
-                             collection))
-           (ido-selected (ido-completing-read prompt ido-collection)))
-      (if (and ido-selected action)
-          (funcall action
-                   (if list-of-pair
-                       (cdar (delq nil
-                                   (mapcar (lambda (x)
-                                             (and (string= (car x)
-                                                           ido-selected)
-                                                  x))
-                                           collection)))
-                     ido-selected)))
-      ido-selected))
-   (t
-    (ivy-read prompt collection
-              :action action))))
+  (let* (selected)
+    (cond
+     ((and action (= 1 (length collection)))
+      ;; select the only candidate immediately
+      (setq selected (car collection)))
 
-(defun ffip-create-shell-command (keyword is-finding-directory)
+     (t
+      (setq selected (completing-read prompt collection))
+      (setq selected (or (assoc selected collection) selected))))
+
+    (when selected
+      ;; make sure only the string/file is passed to action
+      (let* ((default-directory (ffip-get-project-root-directory)))
+        (funcall action (if (consp selected) (cdr selected) selected))))))
+
+(defun ffip-create-shell-command (keyword find-directory-p)
   "Produce command to search KEYWORD.
-If IS-FINDING-DIRECTORY is t, we look up directory instead of file.
+If FIND-DIRECTORY-P is t, we look up directory instead of file.
 Rust fd use regular expression.
 BSD/GNU Find use glob pattern."
   (let* (cmd fmt tgt)
@@ -669,7 +608,7 @@ BSD/GNU Find use glob pattern."
       (setq tgt (if keyword (format "\".*%s\"" keyword) "")))
      (t
       (setq tgt
-            (if is-finding-directory (format "-iwholename \"*%s\"" keyword)
+            (if find-directory-p (format "-iwholename \"*%s\"" keyword)
               (ffip--create-filename-pattern-for-gnufind keyword)))
       (setq fmt (concat "%s "
                         ffip-find-pre-path-options
@@ -678,7 +617,7 @@ BSD/GNU Find use glob pattern."
     (setq cmd (format fmt
                       (ffip--executable-find)
                       (ffip--prune-patterns)
-                      (if is-finding-directory "d" "f")
+                      (if find-directory-p "d" "f")
                       (ffip--ignore-file-names)
                       ffip-find-options
                       (ffip--join-patterns ffip-patterns)
@@ -691,8 +630,14 @@ BSD/GNU Find use glob pattern."
   (setq s (replace-regexp-in-string "\*" ".*" s))
   s)
 
+(defmacro ffip-push-one-candidate (file result)
+  "Push FILE into RESULT."
+  ;; @see https://www.murilopereira.com/how-to-open-a-file-in-emacs/
+  ;; also @see #15 improving handling of directories containing space
+  `(push (cons (replace-regexp-in-string "^\./" "" ,file) ,file) ,result))
+
 ;;;###autoload
-(defun ffip-project-search (keyword is-finding-directory &optional directory-to-search)
+(defun ffip-project-search (keyword &optional find-directory-p)
   "Return an alist of all filenames in the project and their path.
 
 Files with duplicate filenames are suffixed with the name of the
@@ -700,27 +645,30 @@ directory they are found in so that they are unique.
 
 If KEYWORD is string, it's the file name or file path to find file.
 If KEYWORD is list, it's the list of file names.
-IF IS-FINDING-DIRECTORY is t, we are searching directories, else files.
-DIRECTORY-TO-SEARCH specify the root directory to search."
-  (let* ((root (or directory-to-search
-                   (ffip-get-project-root-directory)))
-         (default-directory (file-name-as-directory root))
-         (cmd (ffip-create-shell-command keyword is-finding-directory))
-         (collection (split-string (ffip-shell-command-to-string cmd) "[\r\n]+" t)))
+IF FIND-DIRECTORY-P is t, we are searching directories, else files."
+  (let* ((default-directory (ffip-get-project-root-directory))
+         (cmd (ffip-create-shell-command keyword find-directory-p))
+         (fd-file-pattern (concat "^" (mapconcat 'ffip-glob-to-regex ffip-patterns "\\|") "$"))
+         (collection (split-string (ffip-shell-command-to-string cmd) "[\r\n]+" t))
+         rlt)
 
     (if ffip-debug (message "run command at %s: %s" default-directory cmd))
 
-    (when (and ffip-use-rust-fd ffip-patterns)
-      ;; filter result with Lisp because fd does NOT support multiple patterns
-      (let* ((r (concat "^" (mapconcat 'ffip-glob-to-regex ffip-patterns "\\|") "$")))
-        (setq collection (delq nil (mapcar (lambda (s)
-                                             (if (string-match-p r s) s))
-                                           collection)))))
-    (mapcar (lambda (file)
-              (cons (replace-regexp-in-string "^\./" "" file)
-                    (expand-file-name file)))
-            ;; #15 improving handling of directories containing space
-            collection)))
+    ;; use simple loop statement for clean code
+    (cond
+     ((and ffip-use-rust-fd ffip-patterns)
+      (let* ((fd-file-pattern (concat "^"
+                                      (mapconcat 'ffip-glob-to-regex ffip-patterns "\\|")
+                                      "$")))
+        (dolist (file collection)
+          ;; filter result with Lisp because fd does NOT support multiple patterns
+          (if (string-match fd-file-pattern file) (ffip-push-one-candidate file rlt)))))
+
+     (t
+      (dolist (file collection)
+        (ffip-push-one-candidate file rlt))))
+
+    (nreverse rlt)))
 
 (defun ffip--forward-line (lnum)
   "Forward LNUM lines."
@@ -729,30 +677,34 @@ DIRECTORY-TO-SEARCH specify the root directory to search."
     (goto-char (point-min))
     (forward-line (1- lnum))))
 
+(defun ffip-hint ()
+  "Hint."
+  (let ((root (ffip-get-project-root-directory)))
+    (format "Find in %s/: "
+            (file-name-nondirectory (directory-file-name root)))))
+
 ;;;###autoload
-(defun ffip-find-files (keyword open-another-window &optional find-directory fn)
+(defun ffip-find-files (keyword open-another-window &optional find-directory-p fn)
   "Use KEYWORD to find files.
 If OPEN-ANOTHER-WINDOW is t, the results are displayed in a new window.
-If FIND-DIRECTORY is t, only search directories.  FN is callback.
+If FIND-DIRECTORY-P is t, only search directories.  FN is callback.
 This function is the API to find files."
-  (let* (cands lnum file root)
+  (let* (cands lnum file)
     ;; extract line num if exists
     (when (and keyword (stringp keyword)
                (string-match "^\\(.*\\):\\([0-9]+\\):?$" keyword))
       (setq lnum (string-to-number (match-string 2 keyword)))
       (setq keyword (match-string 1 keyword)))
 
-    (setq cands (ffip-project-search keyword find-directory))
+    (setq cands (ffip-project-search keyword find-directory-p))
     (cond
      ((> (length cands) 0)
-      (setq root (file-name-nondirectory (directory-file-name (ffip-get-project-root-directory))))
       (ffip-completing-read
-       (format "Find in %s/: " root)
+       (ffip-hint)
        cands
        `(lambda (file)
           ;; only one item in project files
-          (if (listp file) (setq file (cdr file)))
-          (if ,find-directory
+          (if ,find-directory-p
               (if (quote ,open-another-window)
                   (dired-other-window file)
                 (switch-to-buffer (dired file)))
@@ -933,7 +885,7 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
 
 ;;;###autoload
 (defun find-file-in-current-directory (&optional level)
-  "Search fil in current directory or LEVEL up parent directory."
+  "Search file in current directory or LEVEL up parent directory."
   (interactive "P")
   (unless level (setq level 0))
   (let* ((ffip-project-root (ffip-parent-directory level default-directory)))
@@ -965,17 +917,12 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
   "Insert contents of file in current buffer.
 The file name is selected interactively from candidates in project."
   (interactive)
-  (let* ((cands (ffip-project-search (ffip-read-keyword) nil))
+  (let* ((cands (ffip-project-search (ffip-read-keyword)))
          root)
     (when (> (length cands) 0)
-      (setq root (file-name-nondirectory (directory-file-name (ffip-get-project-root-directory))))
-      (ffip-completing-read
-       (format "Read file in %s/: " root)
-       cands
-       `(lambda (file)
-          ;; only one item in project files
-          (if (listp file) (setq file (cdr file)))
-          (insert-file file))))))
+      (ffip-completing-read (ffip-hint)
+                            cands
+                            'insert-file))))
 
 ;;;###autoload
 (defun find-file-with-similar-name (&optional open-another-window)
@@ -1000,29 +947,26 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
     (find-file-in-project-by-selected open-another-window)))
 
 ;;;###autoload
-(defun ffip-find-relative-path(&optional find-directory)
+(defun ffip-find-relative-path(&optional find-directory-p)
   "Find file/directory and copy its relative path into `kill-ring'.
-If FIND-DIRECTORY is t, copy the directory path.
+If FIND-DIRECTORY-P is t, copy the directory path.
 
 Set `ffip-find-relative-path-callback' to format the result,
   (setq ffip-find-relative-path-callback 'ffip-copy-reactjs-import)
   (setq ffip-find-relative-path-callback 'ffip-copy-org-file-link)"
   (interactive "P")
-  (let* ((cands (ffip-project-search (ffip-read-keyword) find-directory))
-         root)
+  (let* ((cands (ffip-project-search (ffip-read-keyword) find-directory-p)))
     (cond
      ((> (length cands) 0)
-      (setq root (file-name-nondirectory (directory-file-name (ffip-get-project-root-directory))))
       (ffip-completing-read
-       (format "Find in %s/: " root)
+       (ffip-hint)
        cands
-       `(lambda (p)
+       `(lambda (file)
           ;; only one item in project files
-          (if (listp p) (setq p (cdr p)))
-          (if ,find-directory
-              (setq p (file-name-as-directory p)))
-          (setq p (file-relative-name p (file-name-directory buffer-file-name)))
-          (funcall ffip-find-relative-path-callback p))))
+          (if ,find-directory-p
+              (setq file (file-name-as-directory file)))
+          (setq file (file-relative-name file (file-name-directory buffer-file-name)))
+          (funcall ffip-find-relative-path-callback file))))
      (t
       (message "Nothing found!")))))
 
@@ -1053,7 +997,7 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
 ;;;###autoload
 (defun ffip-lisp-find-file-in-project (&optional directory-p)
   "If DIRECTORY-P is nil, find file in project, or else find directory.
-It's is written in pure Lisp, so it works in all environments."
+This command works in any environment (Windows, etc) out of box."
   (interactive "P")
   (let* ((root (ffip-get-project-root-directory))
          (input-regex (read-string "Input regex (or press ENTER): "))
@@ -1079,59 +1023,19 @@ It's is written in pure Lisp, so it works in all environments."
     (ffip-completing-read
      (format "%s %s: " (if directory-p "directories" "files") root)
      cands
-     `(lambda (item)
+     `(lambda (file)
         (if ,directory-p
-            (switch-to-buffer (dired item))
-          (find-file item))))))
+            (switch-to-buffer (dired file))
+          (find-file file))))))
 
 ;;;###autoload
 (defalias 'ffip 'find-file-in-project)
 (defalias 'find-relative-path 'ffip-find-relative-path)
 
 (defun ffip-path (candidate)
-  "Get path from ivy CANDIDATE."
+  "Get path from CANDIDATE."
   (let* ((default-directory (ffip-project-root)))
-    (file-truename (if (consp candidate) (cdr candidate)
-                     candidate))))
-
-(defun ffip-split-window-api (split-fn mv-fn ratio)
-  "Use SPLIT-FN to split window and focus on new window by MV-FN.
-Window split in RATIO."
-  (let* (ratio-val
-         (keyword (if ffip-split-window-without-asking-for-keyword ""
-                    (ffip-read-keyword)))
-         (cands (ffip-project-search keyword nil))
-         (file (if (= 1 (length cands)) (ffip-path (car cands))
-                 (ffip-path (ffip-completing-read "Find file: " cands))))
-         (buf (if (and file (file-exists-p file)) (find-file-noselect file)
-                (other-buffer))))
-    (cond
-     (ratio
-      (setq ratio-val (cdr (assoc ratio ffip-window-ratio-alist)))
-      (funcall split-fn (floor (/ (window-body-width)
-                                  (1+ ratio-val)))))
-     (t
-      (funcall split-fn)))
-    (set-window-buffer (next-window) buf)
-    (if (or (not ratio-val)
-            (>= ratio-val 1))
-        (funcall mv-fn))))
-
-;;;###autoload
-(defun ffip-split-window-horizontally (&optional ratio)
-  "Find&Open file in horizontal split window.
-New window size is looked up in `ffip-window-ratio-alist' by RATIO.
-Keyword to search new file is selected text or user input."
-  (interactive "P")
-  (ffip-split-window-api 'split-window-horizontally 'windmove-right ratio))
-
-;;;###autoload
-(defun ffip-split-window-vertically (&optional ratio)
-  "Find&Open file in vertical split window.
-New window size is looked up in `ffip-window-ratio-alist' by RATIO.
-Keyword to search new file is selected text or user input."
-  (interactive "P")
-  (ffip-split-window-api 'split-window-vertically 'windmove-down ratio))
+    (file-truename (if (consp candidate) (cdr candidate) candidate))))
 
 ;;;###autoload
 (defun ffip-diff-quit ()
@@ -1145,10 +1049,11 @@ Keyword to search new file is selected text or user input."
   "File file(s) in current hunk.
 If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
   (interactive "P")
-  (let* ((files (mapcar 'file-name-nondirectory (diff-hunk-file-names)))
+  (let* ((files (mapcar (lambda (f) (replace-regexp-in-string "^[^/]*/" "" f)) (diff-hunk-file-names)))
          (alnum 0)
          (blnum 0)
-         (regex "\\(?:\\*\\{15\\}.*\n\\)?[-@* ]*\\([0-9,]+\\)\\([ acd+]+\\([0-9,]+\\)\\)?"))
+         (regex "\\(?:\\*\\{15\\}.*\n\\)?[-@* ]*\\([0-9,]+\\)\\([ acd+]+\\([0-9,]+\\)\\)?")
+         (ffip-match-path-instead-of-filename t))
 
     (save-excursion
       (diff-beginning-of-hunk t)
@@ -1294,9 +1199,9 @@ If NUM is not nil, the corresponding backend is executed directly."
       (ffip-completing-read
        "Run diff backend:"
        descriptions
-       `(lambda (d)
-          (if (string-match "^\\([0-9]+\\): " d)
-              (ffip-show-diff-internal (string-to-number (match-string 1 d))))))))))
+       (lambda (file)
+         (if (string-match "^\\([0-9]+\\): " file)
+             (ffip-show-diff-internal (string-to-number (match-string 1 file))))))))))
 
 (defalias 'ffip-show-diff 'ffip-show-diff-by-description)
 
@@ -1309,7 +1214,7 @@ If NUM is not nil, the corresponding backend is executed directly."
     (let* ((args (ad-get-args 0))
            (file-name (file-name-nondirectory (nth 2 args)))
            (default-directory (ffip-project-root))
-           (cands (ffip-project-search file-name nil default-directory))
+           (cands (ffip-project-search file-name))
            (rlt (if cands (ffip-completing-read "Files: " cands))))
       (when rlt
         (setq rlt (file-truename rlt))
@@ -1333,6 +1238,43 @@ If REVERSE is t, applied patch is reverted."
    (t
     (message "This command only run in `diff-mode' and `ffip-diff-mode'."))))
 
+;;;###autoload
+(defun ffip-fix-file-path-at-point (&optional absolute-path-p)
+  "Fix file path at point.
+If ABSOLUTE-PATH-P is t, old path is replaced by correct absolute path.
+Or else it's replaced by relative path."
+  (interactive "P")
+  (let* ((fn (thing-at-point 'filename))
+         full-path
+         cands)
+    (cond
+     ((not fn)
+      (message "There is no file path at point."))
+
+     ;; path at point is a path of physical file
+     ((setq full-path (ffip--guess-physical-path fn))
+      nil)
+
+     ;; find a file
+     ((setq cands (ffip-project-search (replace-regexp-in-string ffip-relative-path-pattern "" fn)))
+      (cond
+       ((eq (length cands) 1)
+        (setq full-path (nth 0 cands)))
+       (t
+        (ffip-completing-read "Find file: "
+                              cands
+                              `(lambda (file)
+                                 (setq ,full-path file)))))))
+
+    (when full-path
+      (if (consp full-path) (setq full-path (cdr full-path)))
+      (let* ((bounds (bounds-of-thing-at-point 'filename))
+             (path (if absolute-path-p full-path
+                     (file-relative-name full-path))))
+        (goto-char (car bounds))
+        (delete-region (car bounds) (cdr bounds))
+        (insert (replace-regexp-in-string "/index\\.[jt]s" "" path))))))
+
 ;; safe locals
 (progn
   (put 'ffip-diff-backends 'safe-local-variable 'listp)
