@@ -24,15 +24,6 @@
 (require 'ert)
 (require 'find-file-in-project)
 
-(defvar ivy-read-called nil)
-
-(cl-defun ivy-read (prompt collection
-                           &key predicate require-match initial-input
-                           history preselect keymap update-fn sort
-                           action unwind re-builder matcher dynamic-collection caller)
-  (setq ivy-read-called t)
-  (message "`ivy-read' mockup is called"))
-
 (defun get-full-path (filename)
   (concat
    (if load-file-name (file-name-directory load-file-name) default-directory)
@@ -41,7 +32,7 @@
 (ert-deftest ffip-test-find-by-selected ()
   (let (files)
     (setq ffip-project-root default-directory)
-    (setq files (mapcar 'car (ffip-project-search "git-diff" nil)))
+    (setq files (mapcar 'car (ffip-project-search "git-diff")))
     ;; (message "files=%s" files)
     (should (string-match-p "git-diff.diff" (car files)))))
 
@@ -49,7 +40,7 @@
 (ert-deftest ffip-test-ffip ()
   (let (files)
     (setq ffip-project-root default-directory)
-    (setq files (mapcar 'car (ffip-project-search nil nil)))
+    (setq files (mapcar 'car (ffip-project-search nil)))
     (should (> (length files) 1))
     (should (not (active-minibuffer-window)))))
 
@@ -58,7 +49,7 @@
         (prefix-args '(4 (4))))
     (dolist (open-another-arg prefix-args)
       (setq ffip-project-root default-directory)
-      (setq files (mapcar 'car (ffip-project-search "git-diff" nil)))
+      (setq files (mapcar 'car (ffip-project-search "git-diff")))
       (should (= (length files) 1))
       (should (not (active-minibuffer-window))))))
 
@@ -71,19 +62,15 @@
     (switch-to-buffer "*ffip-diff*")
     (goto-char (point-min))
     (diff-file-next)
-    (setq ivy-read-called nil)
     ;; find now
     (ffip-diff-find-file)
-    (should (not ivy-read-called)) ; only one candidate
     (should (string= (file-name-nondirectory (buffer-file-name)) "ffip-tests.el"))
 
     ;; move to the second file hunk
     (switch-to-buffer "*ffip-diff*")
     (diff-file-next)
-    (setq ivy-read-called nil)
     ;; find file in the first diff hunk now
     (ffip-diff-find-file)
-    (should (not ivy-read-called)) ; only one candidate
     (should (string= (file-name-nondirectory (buffer-file-name)) "git-diff.diff"))
     ;; cleanup
     (kill-buffer "*ffip-diff*")))
@@ -97,11 +84,33 @@
     (should (string= (ffip-parent-directory 4 dir) "/"))
     (should (string= (ffip-parent-directory 999 dir) "/"))))
 
+(ert-deftest ffip-test-guess-physical-path ()
+  (let* (fn
+         (default-directory (file-name-directory (or load-file-name buffer-file-name))))
+    (with-temp-buffer
+      (insert "import './test1';\n")
+      (insert "import './test2';\n")
+      (js-mode) ; javascript
+
+      ;; detect "test1.ts"
+      (goto-char (point-min))
+      (search-forward "test1")
+      (setq fn (ffip-guess-file-name-at-point))
+      (should (string= fn "./test1"))
+      (should (string= (ffip--guess-physical-path fn) (file-truename "./test1.ts")))
+
+      ;; detect "test2.js"
+      (goto-char (point-min))
+      (search-forward "test2")
+      (setq fn (ffip-guess-file-name-at-point))
+      (should (string= fn "./test2"))
+      (should (string= (ffip--guess-physical-path fn) (file-truename "./test2.js"))))
+    ))
+
 (ert-deftest ffip-test-windows ()
-  (let (rlt)
-    (if (eq system-type 'windows-nt)
-        (should (executable-find (ffip--guess-gnu-find)))
-      (message "NOT windows native Emacs, nothing to test.")
-      (should t))))
+  (if (eq system-type 'windows-nt)
+      (should (executable-find (ffip--guess-gnu-find)))
+    (message "NOT windows native Emacs, nothing to test.")
+    (should t)))
 
 (ert-run-tests-batch-and-exit)
\ No newline at end of file
